/*
ID: espr1t
TASK: Closest Points
KEYWORDS: Testgen
*/

#include <cstdio>
#include <cstdlib>

const int MAX = 100000;
const int DIM = 5;
const int MIN_COORD = -1000000;
const int MAX_COORD = +1000000;

int rand30() {
    return (((rand() & (((1 << 10) - 1) << 3)) >> 3) <<  0) |
           (((rand() & (((1 << 10) - 1) << 3)) >> 3) << 10) |
           (((rand() & (((1 << 10) - 1) << 3)) >> 3) << 20) ;
}

int n, k, q;
int a[MAX][DIM], b[MAX][DIM];

void genTest(int n_, int k_, int q_, int minCoord, int maxCoord) {
    // Comment these two lines to get random, but limited in space tests
    minCoord = MIN_COORD;
    maxCoord = MAX_COORD;
    
    n = n_, k = k_, q = q_;
    for (int i = 0; i < n; i++) {
        for (int c = 0; c < k; c++) {
            a[i][c] = rand30() % (maxCoord - minCoord + 1) + minCoord;
        }
    }
    for (int i = 0; i < q; i++) {
        for (int c = 0; c < k; c++)
            b[i][c] = rand30() % (maxCoord - minCoord + 1) + minCoord;
    }
}

void sampleInput() {
    n = 11; k = 4;
    int coords[11][4] = {
        {7, 1, 19, 5},
        {18, 6, 15, 0},
        {17, 5, 6, 4},
        {10, 1, 5, 20},
        {20, 20, 8, 20},
        {3, 18, 13, 8},
        {13, 5, 15, 1},
        {1, 19, 5, 6},
        {13, 15, 10, 4},
        {15, 16, 19, 4},
        {13, 5, 15, 1},
    };
    for (int i = 0; i < n; i++)
        for (int dim = 0; dim < k; dim++)
            a[i][dim] = coords[i][dim] - 10;
    
    q = 5;
    int queries[5][4] = {
        {1, 19, 1, 9},
        {17, 12, 2, 17},
        {16, 15, 1, 2},
        {17, 12, 2, 17},
        {10, 1, 5, 20}
    };
    for (int i = 0; i < q; i++)
        for (int dim = 0; dim < k; dim++)
            b[i][dim] = queries[i][dim] - 10;
}

void printTest(int testNum) {
    char fileName[32];
    sprintf(fileName, "ClosestPoints.%02d.in", testNum);
    FILE* out = fopen(fileName, "wt");

    if (n < 1 || n > MAX) {
        fprintf(stderr, "n should be in [%d, %d], but is %d.\n", 1, MAX, n);
        exit(1);
    }
    if (k < 1 || k > DIM) {
        fprintf(stderr, "k should be in [%d, %d], but is %d.\n", 1, DIM, k);
        exit(1);
    }
    if (q < 1 || q > MAX) {
        fprintf(stderr, "q should be in [%d, %d], but is %d.\n", 1, MAX, q);
        exit(1);
    }
    
    for (int i = 0; i < n; i++) {
        for (int d = 0; d < k; d++) {
            if (a[i][d] < MIN_COORD || a[i][d] > MAX_COORD) {
                fprintf(stderr, "coordinates should be in [%d, %d], but is %d.\n",
                    MIN_COORD, MAX_COORD, a[i][d]);
                exit(1);
            }
        }
    }
    for (int i = 0; i < q; i++) {
        for (int d = 0; d < k; d++) {
            if (b[i][d] < MIN_COORD || b[i][d] > MAX_COORD) {
                fprintf(stderr, "coordinates should be in [%d, %d], but is %d.\n",
                    MIN_COORD, MAX_COORD, b[i][d]);
                exit(1);
            }
        }
    }
    

    fprintf(out, "%d %d\n", n, k);
    for (int i = 0; i < n; i++)
        for (int d = 0; d < k; d++)
            fprintf(out, "%d%c", a[i][d], d + 1 == k ? '\n' : ' ');
    fprintf(out, "%d\n", q);
    for (int i = 0; i < q; i++)
        for (int d = 0; d < k; d++)
            fprintf(out, "%d%c", b[i][d], d + 1 == k ? '\n' : ' ');
    
    fclose(out);
}

int main(void) {
    srand(42);
    
    int numTests = 20;
    for (int test = 0; test <= numTests; test++) {
        fprintf(stderr, "Generating test %d...\n", test);
        if (test == 0) sampleInput();
        else if (test == 1) genTest(100, 1, 10, -100, 100);
        else if (test == 2) genTest(10000, 1, 10000, 900000, 910000);
        else if (test == 3) genTest(100000, 1, 100000, 0, 100000);
        else if (test == 4) genTest(100000, 1, 100000, -1000000, 1000000);
        else if (test == 5) genTest(1, 2, 100000, -100000, 1000000);
        else if (test == 6) genTest(2000, 2, 1000, 0, 5000);
        else if (test == 7) genTest(10000, 2, 20000, -1000000, 1000000);
        else if (test == 8) genTest(50000, 2, 50000, -1000000, -999500);
        else if (test == 9) genTest(100000, 2, 100000, -10000, 10000);
        else if (test == 10) genTest(100000, 2, 100000, -1000000, 1000000);
        else if (test == 11) genTest(20000, 3, 20000, -1000, 1000);
        else if (test == 12) genTest(50000, 3, 50000, -1000000, 1000000);
        else if (test == 13) genTest(100000, 3, 100000, 0, 100000);
        else if (test == 14) genTest(30000, 4, 30000, 0, 100);
        else if (test == 15) genTest(60000, 4, 60000, 0, 100000);
        else if (test == 16) genTest(90000, 4, 90000, -1000000, 1000000);
        else if (test == 17) genTest(40000, 5, 40000, 0, 20000);
        else if (test == 18) genTest(60000, 5, 60000, 0, 1000);
        else if (test == 19) genTest(80000, 5, 80000, -100000, 100000);
        else if (test == 20) genTest(100000, 5, 100000, -1000000, 1000000);
        else fprintf(stderr, "DAFUQ?!\n");
        printTest(test);
    }
    fprintf(stderr, "Generated all tests successfully.\n");
    return 0;
}
